%FreakLab - FreakLab_opt1.m
%
%Erstellt von Matthias Hasewend im Rahmen der bung 'Applikationssoftware
%fr Fortgeschrittene' bei Ass. Prof. Winfried Kernbichler, Institut fr
%Theoretische Physik / Computational Physics an der TU Graz im WS 2010/11.
%
%Beschreibung:
%
%Dieses Skript steuert das Bedienfeld 1 - Lineare Abbildung.
%
%Theorie:
%
%Jedem Pixel eines Bildes entspicht ein Element eines 2D-Vektorraumes.
%Isomorphismen zwischen zwei solchen Rum knnen durch 2 x 2 Matrizen
%dargestellt werden. Dementsprechend kann eine Matrix auf ein Bild eine
%Lineare Transformation ausben. Dies kann Stauchungen, Steckungen,
%Verdrehungen oder Spiegelungen des Bildes bedeuten. Auch Mischungen dieser
%Elementaroperationen sind mglich. In diesem Bedienfeld kann der Benutzer
%verschiedene Elemtaroperationen ausben, deren Strke durch Slider
%regelbar ist. Die zugehrige Abbildungsmatrix wird angezeigt und kann auch
%modifiziert werden, sodass auch allgemeinere Mischabbildungen mglich
%sind.

%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

%Diese Funktion bewirkt, dass das alle anderen Funktionen in diesem Skript 
%von aussen aufgerufen werden knnen in der Form: FreakLab_opt1('Funkname')
function varargout = FreakLab_opt1(varargin)

    func = str2func(varargin{1});
    
    if(nargout)
        
        [varargout{1:nargout}] = func(varargin{2:end});
        
    else
        
        func(varargin{2:end});
        
    end
    
end

%Aktion beim Einstellen der 'Drehstrke' ber den Slider
function slider1_Callback(hObject, eventdata, handles)

    %Einlesen des Sliderwerts 0 bis 1
    inp = get(hObject,'Value');
    %Umrechnen auf -180 bis +180
    inp = 360*inp - 180;
    
    %zug. Transformationsmatrix errechnen
    handles.afftra = [cosd(inp),-sind(inp);sind(inp),cosd(inp)];
    %Transformation mit Hilfsfunktion am Preview durchfhren
    matupdate(handles,[inp,0,0,0],[0,0,0],2*ones(1,7));
    %Preview anzeigen
    disp_1_1(hObject,handles);

end

%Aktion beim Einstellen der 'vertikalen Scherstrke' ber den Slider
function slider2_Callback(hObject, eventdata, handles)

    inp = get(hObject,'Value');
    
    inp = 2*inp - 1;
    
    handles.afftra = [1,inp;0,1];
    matupdate(handles,[0,inp,0,0],[0,0,0],2*ones(1,7));
    disp_1_1(hObject,handles);
    
end

%Aktion beim Einstellen der 'horizontalen Scherstrke' ber den Slider
function slider3_Callback(hObject, eventdata, handles)

    inp = get(hObject,'Value');
    
    inp = 2*inp - 1;
    
    handles.afftra = [1,0;inp,1];
    matupdate(handles,[0,0,inp,0],[0,0,0],2*ones(1,7));
    disp_1_1(hObject,handles);
    
end

%Aktion beim Einstellen der 'diagonalen Scherstrke' ber den Slider
function slider4_Callback(hObject, eventdata, handles)

    inp = get(hObject,'Value');
    
    inp = 1.8*inp - 0.9;
    
    handles.afftra = [1,inp;inp,1];
    matupdate(handles,[0,0,0,inp],[0,0,0],2*ones(1,7));
    disp_1_1(hObject,handles);
    
end

%Man beachte: Alle Edit-Textfelder wurden unter init mit der
%KeyPressFcn editupdate versehen. Diese bewirkt ein Update des
%Previews schon whrend der Eingabe (siehe editupdate.m)

%Aktion bei Eingabe der 'Drehstrke' ber das Textfeld
function edit1_Callback(hObject, eventdata, handles)

    %Einlesen und Checken der Eingabe am Textfeld. Bei berschreiten der
    %Grenzwerte wird auf den oberen / unteren Grenzwert +180 / -180
    %zurckgesetzt, bei fehlerhafter Eingabe auf den Defaultwert 0.
    inp = inpcheck(str2num(get(hObject, 'String')),0,-180,180);

    %zug. Transformationsmatrix errechnen
    handles.afftra = [cosd(inp),-sind(inp);sind(inp),cosd(inp)];
    
    %Transformation mit Hilfsfunktion am Preview durchfhren
    matupdate(handles,[inp,0,0,0],[0,0,0],2*ones(1,7));
    %Preview anzeigen
    disp_1_1(hObject,handles);
    
end

%Aktion bei Eingabe der 'vertikalen Scherstrke' ber das Textfeld
function edit2_Callback(hObject, eventdata, handles)

    %Weil in diesem und den folgenden Textfeldern Kommazahlen eingegeben
    %werden knnen, mssen, um das Update des Previews schon whrend der 
    %Eingabe mglich zu machen, einige Vorkehrungen getroffen werden
    
    %Zunchst nur einlesen und noch kein Check
    inp = get(hObject,'String');
    %Lnge messen
    len = length(inp);
    
    if(isempty(inp))
    
        %Wenn der Benutzer die komplette Zahl entfernt, wird der
        %Default-Wert gesetzt. Dadurch wird verhindert, das leere
        %Textfelder auftauchen
        set(hObject,'String',num2str(0,['%1.2f']));

    %Nur bei nichtleerer Eingabe gibts das Update des Previews und eine
    %Korrektur der Eingabe. Dies Funktioniert aber nur, wenn das letzte
    %eingegebene Zeichen kein 'Kommapunkt' ist
    elseif(inp(end) ~= '.')
        
        %Fr die Korrektur wird die Zahl auf eine gnstige Lnge an 
        %Stellen zurechtgeschnitten. Die gnstige lnge ist um 1 geringer,
        %wenn das Vorzeichen der Zahl negativ ist
        len = len - (inp(1) == '-');

        %Nun kann der Check durchgefhrt ...
        inp = inpcheck(str2num(inp),0,-1,1);
        
        %... und die Transformationsmatrix aktualisiert werden
        handles.afftra = [1,inp;0,1];
        
        %Diese Hilfsvariable sagt der Hilfsfunktion, auf wie viele
        %Nachkommastellen sie die Zahlen in den sieben Textboxen anzeigen 
        %soll (Standard 2). Beim aktuellen Textfeld ist (len >= 3) + (len
        %>= 4) gleich 1 bei Eingabe z.B. 1.2 und gleich zwei bei z.B. 1.23.
        %Weitere Hinweise siehe matupdate.m
        help = [(len >= 3) + (len >= 4),2,2,2,2,2,2];
        %Transformation mit Hilfsfunktion am Preview durchfhren
        matupdate(handles,[0,inp,0,0],[0,0,0],help);
        %Preview anzeigen
        disp_1_1(hObject,handles);
    
    end
    
end

%Aktion bei Eingabe der 'horizontalen Scherstrke' ber das Textfeld
function edit3_Callback(hObject, eventdata, handles)

    inp = get(hObject,'String');
    len = length(inp);
    
    if(isempty(inp))
    
        set(hObject,'String',num2str(0,['%1.2f']));
    
    elseif(inp(end) ~= '.')
        
        len = len - (inp(1) == '-');

        inp = inpcheck(str2num(inp),0,-1,1);
        
        handles.afftra = [1,0;inp,1];
        help = [2,(len >= 3) + (len >= 4),2,2,2,2,2];
        matupdate(handles,[0,0,inp,0],[0,0,0],help);
        disp_1_1(hObject,handles);
    
    end
    
end

%Aktion bei Eingabe der 'diagonalen Scherstrke' ber das Textfeld
function edit4_Callback(hObject, eventdata, handles)

    inp = get(hObject,'String');
    len = length(inp);
    
    if(isempty(inp))
    
        set(hObject,'String',num2str(0,['%1.2f']));
    
    elseif(inp(end) ~= '.')
        
        len = len - (inp(1) == '-');

        inp = inpcheck(str2num(inp),0,-0.9,0.9);
        
        handles.afftra = [1,inp;inp,1];
        help = [2,2,(len >= 3) + (len >= 4),2,2,2,2];
        matupdate(handles,[0,0,0,inp],[0,0,0],help);
        disp_1_1(hObject,handles);
    
    end

end

%Aktion bei Eingabe des 11-Matrixelements ber das Textfeld
function edit5_Callback(hObject, eventdata, handles)

    inp = get(hObject,'String');
    len = length(inp);
    
    if(isempty(inp))
    
        set(hObject,'String',num2str(1,['%1.2f']));
    
    elseif(inp(end) ~= '.')
        
        len = len - (inp(1) == '-');
        
        inp = inpcheck(str2num(inp),1,-2,2);
        
        help = handles.afftra(1,1);
        handles.afftra(1,1) = inp;
    
        disp_1_2(hObject,handles,help,[2,2,2,(len >= 3) + (len >= 4),2,2,2]);
        
    end
    
end

%Aktion bei Eingabe des 12-Matrixelements ber das Textfeld
function edit6_Callback(hObject, eventdata, handles)

    inp = get(hObject,'String');
    len = length(inp);
    
    if(isempty(inp))
    
        set(hObject,'String',num2str(0,['%1.2f']));
    
    elseif(inp(end) ~= '.')
        
        len = len - (inp(1) == '-');
        
        inp = inpcheck(str2num(inp),0,-2,2);
        
        help = handles.afftra(1,2);
        handles.afftra(1,2) = inp;
    
        disp_1_2(hObject,handles,help,[2,2,2,2,(len >= 3) + (len >= 4),2,2]);
        
    end
    
end

%Aktion bei Eingabe des 21-Matrixelements ber das Textfeld
function edit7_Callback(hObject, eventdata, handles)

    inp = get(hObject,'String');
    len = length(inp);
    
    if(isempty(inp))
    
        set(hObject,'String',num2str(0,['%1.2f']));
    
    elseif(inp(end) ~= '.')
        
        len = len - (inp(1) == '-');
        
        inp = inpcheck(str2num(inp),0,-2,2);
        
        help = handles.afftra(2,1);
        handles.afftra(2,1) = inp;
    
        disp_1_2(hObject,handles,help,[2,2,2,2,2,(len >= 3) + (len >= 4),2]);
        
    end
    
end

%Aktion bei Eingabe des 22-Matrixelements ber das Textfeld
function edit8_Callback(hObject, eventdata, handles)

    inp = get(hObject,'String');
    len = length(inp);
    
    if(isempty(inp))
    
        set(hObject,'String',num2str(1,['%1.2f']));
    
    elseif(inp(end) ~= '.')
        
        len = len - (inp(1) == '-');
        
        inp = inpcheck(str2num(inp),1,-2,2);
        
        help = handles.afftra(2,2);
        handles.afftra(2,2) = inp;
    
        disp_1_2(hObject,handles,help,[2,2,2,2,2,2,(len >= 3) + (len >= 4)]);
        
    end
    
end

%Aktion beim vertikalen Spiegeln
function checkbox1_Callback(hObject, eventdata, handles)

    %Die jeweils anderen Checkboxen 'abhackeln'
    set(handles.checkbox2,'Value',0);
    set(handles.checkbox3,'Value',0);
    
    %Wenn Hackerl gesetzt: Trafo-Matrix modifizieren
    if(get(hObject,'Value') == 1)
        
        handles.afftra = [-1,0;0,1];
        
    else
        
        handles.afftra = [1,0;0,1];
        
    end
    
    %Transformation mit Hilfsfunktion am Preview durchfhren
    matupdate(handles,[0,0,0,0],[get(hObject,'Value'),0,0],2*ones(1,7));
    %Preview anzeigen
    disp_1_1(hObject,handles);

end

%Aktion beim horizontalen Spiegeln
function checkbox2_Callback(hObject, eventdata, handles)

	set(handles.checkbox1,'Value',0);
    set(handles.checkbox3,'Value',0);
    
    if(get(hObject,'Value') == 1)
        
        handles.afftra = [1,0;0,-1];
        
    else
        
        handles.afftra = [1,0;0,1];
        
    end
    
    matupdate(handles,[0,0,0,0],[0,get(hObject,'Value'),0],2*ones(1,7));
    disp_1_1(hObject,handles);

end

%Aktion beim diagonalen Spiegeln
function checkbox3_Callback(hObject, eventdata, handles)

	set(handles.checkbox1,'Value',0);
    set(handles.checkbox2,'Value',0);
    
    if(get(hObject,'Value') == 1)
        
        handles.afftra = [-1,0;0,-1];
        
    else
        
        handles.afftra = [1,0;0,1];
        
    end
        
    matupdate(handles,[0,0,0,0],[0,0,get(hObject,'Value')],2*ones(1,7));
    disp_1_1(hObject,handles);

end